using System.Collections.Generic;
using System;
using System.Text;

public class GPDefNode {
  public string Name { get; set; }
  public CodeNode ClassCodeNode { get; set; }
  public CodeNode InitCodeNode { get; set; }
  public List<SymbolNode> NonTerminals { get; private set; }
  public List<SymbolNode> Terminals { get; private set; }
  public List<RuleNode> Rules { get; private set; }
  public FitnessFunctionNode FitnessFunctionNode { get; set; }

  public GPDefNode() {
    this.NonTerminals = new List<SymbolNode>();
    this.Terminals = new List<SymbolNode>();
    this.Rules = new List<RuleNode>();
    this.ClassCodeNode = new CodeNode();
    this.InitCodeNode = new CodeNode();
  }
}

public class CodeNode {
  public string SrcCode { get; set; }
}

public class FitnessFunctionNode {
  public bool Maximization { get; set; }
  public string SrcCode { get; set; }
}

public class SymbolNode {
  public string Ident { get; set; }
  public string FormalParameters { get; set; }

}

public class NonTerminalNode : SymbolNode {
}

public class TerminalNode : SymbolNode {
  public class FieldDefinition {
    private readonly string refOrOut;
    private readonly string type;
    private readonly string identifier;

    public FieldDefinition(string refOrOut, string type, string identifier) {
      this.type = type;
      this.identifier = identifier;
      this.refOrOut = refOrOut;
    }
    public string RefOrOut { get { return this.refOrOut; } }
    public string Type { get { return this.type; } }
    public string Identifier { get { return this.identifier; } }
  }
  public IEnumerable<ConstraintNode> Constraints { get; set; }
  public IEnumerable<FieldDefinition> FieldDefinitions { get; set; }
}


public enum ConstraintNodeType { Set, Range };
public class ConstraintNode {
  public string Ident { get; set; }
  public ConstraintNodeType Type { get; set; }
  public string SetExpression { get; set; }
  public string RangeMinExpression { get; set; }
  public string RangeMaxExpression { get; set; }
  public ConstraintNode(string ident) {
    this.Ident = ident;
  }
}

public class RuleNode {
  public string NtSymbol { get; set; }
  public string LocalCode { get; set; }
  public RuleExprNode RuleExpr { get; set; }
}

public class RuleExprNode { }

public class AlternativesNode : RuleExprNode {
  private List<RuleExprNode> alt;
  public IEnumerable<RuleExprNode> Alternatives { get { return alt; } }
  public AlternativesNode() {
    this.alt = new List<RuleExprNode>();
  }
  public void Add(RuleExprNode expr) {
    alt.Add(expr);
  }
}

public class SequenceNode : RuleExprNode {
  private readonly List<RuleExprNode> seq;
  public IEnumerable<RuleExprNode> Sequence { get { return seq; } }
  public SequenceNode() {
    seq = new List<RuleExprNode>();
  }
  public void Add(RuleExprNode expr) {
    seq.Add(expr);
  }
}

public class CallSymbolNode : RuleExprNode {
  public string Ident { get; set; }
  public string ActualParameter { get; set; }
}

public class RuleActionNode : RuleExprNode {
  public string SrcCode { get; set; }
}